;==============================================================================
;*Author:  Jonathan Weaver, jonw0224@netscape.net
;E-mail Contact: jonw0224@netscape.net
;*Description:  The transmit and recieve code for the software (bitbang) serial port.
;*Version: 1.0
;*Date: 3/30/2005
;*Filename: serial.inc
;
;Copyright (C) 2005 Jonathan Weaver
;
;This program is free software; you can redistribute it and/or modify it under
;the terms of the GNU General Public License as published by the Free Software
;Foundation; either version 2 of the License, or (at your option) any later
;version.
;
;This program is distributed in the hope that it will be useful, but WITHOUT
;ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
;FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
;details.
;
;You should have received a copy of the GNU General Public License along with
;this program; if not, write to the Free Software Foundation, Inc.,
;51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
;
;Revision History:
; v1.0 - 2/16/2005 - Created file, tested modes and timing of UART_Tx
; v1.0 - 3/30/2005 - Completed testing of UART_Tx and UART_Rx.  Added ability
;                    to call UART_Rx from an interrupt routine and added
;                    RXLATENCY and TXLATENCY to UART_Rx and UART_Tx.  UART_Tx
;                    now disables interrupts during transmission.
;;
;TODO:  Test the modes and timing of the recieve code ... done
;
;Additional Testing Set:
;(1)  Transmit, odd and even polarity
;       UART_Tx_DT < 23 ... odd OK, 3/15/2005, even should be OK
;       UART_Tx_DT > 22 ... odd OK, 3/15/2005, even should be OK
;       UART_Tx_DT > 33 ... odd OK, 3/15/2005, even OK 3/16/2005
;(2)  Recieve
;(2.1) Positive Polarity, Odd Parity
;   (a) No error
;       UART_Tx_DT < 17 ... ok, 3/28/05
;       UART_Tx_DT < 19 ... ok, 3/29/05
;       UART_Tx_DT < 30 ... ok, 3/29/05
;       UART_Tx_DT >= 30 ... ok, 3/29/05
;   (b) Parity Error
;       UART_Tx_DT < 17 ... ok, 3/28/05
;       UART_Tx_DT < 19 ... ok, 3/29/05
;       UART_Tx_DT < 30 ... ok, 3/29/05
;       UART_Tx_DT >= 30 ... ok, 3/29/05
;(2.2) Positive Polarity, Even Parity
;   (a) No error
;       UART_Tx_DT < 17 ... ok, 3/28/05
;       UART_Tx_DT < 19 ... ok, 3/29/05
;       UART_Tx_DT < 30 ... ok, 3/29/05
;       UART_Tx_DT >= 30 ... ok, 3/29/05
;   (b) Parity Error
;       UART_Tx_DT < 17 ... ok, 3/28/05
;       UART_Tx_DT < 19 ... ok, 3/29/05
;       UART_Tx_DT < 30 ... ok, 3/29/05
;       UART_Tx_DT >= 30 ... ok, 3/29/05
;(2.3) Positive Polarity, Sticky High Parity
;   (a) No error
;       UART_Tx_DT < 13 ... ok, 3/28/05
;       UART_Tx_DT >= 13 ... ok, 3/29/05
;   (b) Parity Error
;       UART_Tx_DT < 13 ... ok. 3/28/05
;       UART_Tx_DT >= 13 ... ok.  3/29/05
;(2.4) Positive Polarity, Sticky Low Parity
;   (a) No error
;       UART_Tx_DT < 13 ... ok. 3/28/05
;       UART_Tx_DT >= 13 ... ok, 3/29/05
;   (b) Parity Error
;       UART_Tx_DT < 13 ... ok, 3/28/05
;       UART_Tx_DT >= 13 ... ok, 3/29/05
;
;  TODO:  Add capability for recieve to work with an interrupt routine ... done
;         Add transmit latency ... done
;
;  TODO:  Add smart timing code for extremely fast baudrates ...
;
;==============================================================================

;*bitCount is used internally, but must be declared in the main program
;external bitCount

;*rxReg is used internally, but must be declared in the main program
;external rxReg

;Timing variable for use in UART_Rx and UART_Tx
UART_Tx_DT equ CLOCKFREQ*D'250'/BAUDRATE + ((CLOCKFREQ*D'500') % (2*BAUDRATE))/BAUDRATE

;------------------------------------------------------------------------------
ifdef _UART_Rx      ;Compile code if requested
;*Recieves data on an port pin in an asyncronous serial format.  Must define _UART_Rx to compile subroutine
;*rxReg:  a register to place recieved data
;*bitCount:  an internal counting register
;*Returns: status code in WREG
UART_Rx		;public
    if !(SERIAL_CONF & RXINTERRUPT)      ;if not called from an interrupt
        RXSKIPIFZERO
            goto UART_Rx            ;Possible 3 cycle jitter
    endif
    MOVLF 0x08, bitCount
    if (SERIAL_CONF & (PARITYON + STICKYPARITY)) == B'01000000'   ;if even or odd
        DELAY 3*UART_Tx_DT/2 - 6 - RXLATENCY    ;Wait one and a half cycles
        clrw                        ;Prepare for parity calculation
    else
        DELAY 3*UART_Tx_DT/2 - 5 - RXLATENCY   ;Wait one and a half cycles
    endif
UART_Rx_L1                      ;begin loop
    bcf STATUS, C
    RXSKIPIFZERO
        bsf STATUS, C
    rrf rxReg, f
    if (SERIAL_CONF & (PARITYON + STICKYPARITY)) == B'01000000'   ;if even or odd
        if UART_Tx_DT > D'19'
            if UART_Tx_DT > D'30'       ;Slow parity calculation
                xorwf rxReg, w          ;Do parity calculation on MSB of WREG
                addlw B'10000000'       ;move MSB of WREG into CARRY
                rlf bitCount, f         ;Save in least significant bit of bitCount
                DELAY UART_Tx_DT - D'15'
                bcf STATUS, C           ;clear CARRY, move 0 into MSB of bitCount
                rrf bitCount, f         ;Save parity into CARRY, restore bitCount
                clrw                    ;Move parity into MSB of WREG
                btfsc STATUS, C
                    movlw B'10000000'
            else                        ;Delay so that WREG is untouched
                xorwf rxReg, w          ;Do parity calculation on MSB of WREG
                DELAY D'11'             ;Split Delay for code optimization
                DELAY UART_Tx_DT - D'19'
            endif
        else
            xorwf rxReg, w          ;Do parity calculation on MSB of WREG
            DELAY UART_Tx_DT - D'8' ;Wait, but takes one more cycle
        endif
    else
        DELAY UART_Tx_DT - D'7'
    endif
    decfsz bitCount, f          ;continue looping?
        goto UART_Rx_L1         ;end loop
    if SERIAL_CONF & PARITYON               ;if parity is on
        if SERIAL_CONF & STICKYPARITY       ;if parity is sticky
            DELAY 1
            clrw
            if SERIAL_CONF & ODDPARITY      ;if parity is sticky high
                RXSKIPIFONE
                    movlw PEF               ;parity error
            else                            ;parity is sticky low
                RXSKIPIFZERO
                    movlw PEF               ;parity error
            endif
            if UART_Tx_DT > D'13'
                movwf bitCount              ;copy WREG into bitCount
                DELAY UART_Tx_DT - 4        ;Wait for stop bit
                movf bitCount, w            ;copy bitCount back to WREG
            else
                DELAY UART_Tx_DT - 2        ;Wait for stop bit
            endif
        else
            DELAY 2
            if SERIAL_CONF & ODDPARITY      ;if parity is odd
                RXSKIPIFZERO
                    xorlw B'10000000'
                andlw B'10000000'           ;WREG = B'10000000' or B'00000000'
                btfsc STATUS, Z
                    movlw PEF               ;parity error
                andlw B'01111111'           ;clear out MSB of WREG
                if UART_Tx_DT > D'17'
                    movwf bitCount          ;copy WREG into bitCount
                    DELAY UART_Tx_DT - 8    ;Wait for stop bit
                    movf bitCount, w        ;copy bitCount back to WREG
                else
                    DELAY UART_Tx_DT - 6    ;Wait for stop bit
                endif
            else                            ;parity is even
                RXSKIPIFZERO
                    xorlw B'10000000'
                andlw B'10000000'           ;WREG = B'10000000' or B'00000000'
                btfss STATUS, Z
                    movlw PEF               ;parity error
                andlw B'01111111'           ;clear out MSB of WREG
                if UART_Tx_DT > D'17'
                    movwf bitCount          ;copy WREG into bitCount
                    DELAY UART_Tx_DT - 8    ;Wait for stop bit
                    movf bitCount, w        ;Copy bitCount back to WREG
                else
                    DELAY UART_Tx_DT - 6    ;Wait for stop bit
                endif
            endif
        endif
    else
        clrw                    ;Prepare for error code
        DELAY 1                 ;Wait for stop bit
    endif
    RXSKIPIFONE
        xorlw FEF               ;Framing error
    return                      ;Error code in WREG
endif       ;End ifdef

;------------------------------------------------------------------------------
ifdef _UART_Tx          ;Compile code if requested
;*Transmits data on an port pin in an asyncronous serial format.  User must define _UART_Rx to compile subroutine
;*txReg:  a register containing data to be transmitted
;*bitCount:  an internal counting register
;*Returns: status code in WREG
UART_Tx	;public
    bcf INTCON, GIE             ;Interrupts off
    TXZERO                      ;Start Bit
    MOVLF 0x08, bitCount        ;Keep Count
    if (SERIAL_CONF & (PARITYON + STICKYPARITY)) == B'01000000'   ;if even or odd
        DELAY UART_Tx_DT - D'9' ;Wait
        clrw                    ;Prepare for parity calculation
    else
        DELAY UART_Tx_DT - D'7' ;Wait
    endif
UART_Tx_L1                      ;begin loop
    if (SERIAL_CONF & (PARITYON + STICKYPARITY)) == B'01000000'   ;if even or odd
        xorwf txReg, w          ;Parity bit will be determined by LSB of WREG
    endif
    rrf txReg, f                ;Roll transmitting bit to Carry
    btfss STATUS, C             ;Transmit 1 or 0
         goto UART_Tx_L2
    DELAY 1                     ;Get timing right
    TXONE                       ;Transmit one
    goto UART_Tx_L3
UART_Tx_L2
    TXZERO                      ;Transmit zero
    DELAY 2                     ;Get timing right
UART_Tx_L3
    if (SERIAL_CONF & (PARITYON + STICKYPARITY)) == B'01000000'   ;if even or odd
        if UART_Tx_DT > D'22'
            if UART_Tx_DT > D'33'
                andlw B'00000001'       ;WREG = 0 or 1, Z is inversed parity bit
                addlw B'11111111'       ;move parity bit into CARRY
                rlf bitCount, f         ;Save in least significant bit of bitCount
                DELAY UART_Tx_DT - D'19'
                bcf STATUS, C           ;clear CARRY, move 0 into MSB of bitCount
                rrf bitCount, f         ;Save parity into CARRY, restore bitCount
                clrw                    ;Move parity into LSB of WREG
                btfsc STATUS, C
                    movlw B'00000001'
            else
                DELAY D'11'             ;Wait, but parity takes another cycle
                DELAY UART_Tx_DT - D'22';Wait, but parity takes another cycle
            endif
        else
            DELAY UART_Tx_DT - D'11'    ;Wait, but parity takes another cycle
        endif
    else
        DELAY UART_Tx_DT - D'10'    ;Wait
    endif
    decfsz bitCount, f              ;Continue?
        goto UART_Tx_L1             ;end loop
    if SERIAL_CONF & PARITYON               ;if parity is on
        if SERIAL_CONF & STICKYPARITY       ;if parity is sticky
            DELAY D'5'      ;Wait
            if SERIAL_CONF & ODDPARITY      ;if parity is sticky high
                TXONE
            else                            ;parity is sticky low
                TXZERO
            endif
            DELAY UART_Tx_DT - D'1'
        else
            if SERIAL_CONF & ODDPARITY      ;if parity is odd
                DELAY 2
                andlw B'00000001'
                btfss STATUS, Z
                    goto UART_Tx_L4
                DELAY 1
                TXONE
                goto UART_Tx_L5
UART_Tx_L4
                TXZERO
                DELAY 2
UART_Tx_L5
                DELAY UART_Tx_DT - D'3'
            else                            ;parity is even
                DELAY 2
                andlw B'00000001'
                btfss STATUS, Z
                    goto UART_Tx_L4
                DELAY 1
                TXZERO
                goto UART_Tx_L5
UART_Tx_L4
                TXONE
                DELAY 2
UART_Tx_L5
                DELAY UART_Tx_DT - D'3'
            endif
        endif
    else
        DELAY D'5'                          ;Wait
    endif
    TXONE                                   ;Stop Bit
    if SERIAL_CONF & STOPTWO                ;Hold stop bit two cycles
        DELAY UART_Tx_DT*2 - D'5' - TXLATENCY
    else
        if SERIAL_CONF & STOPONEANDHALF     ;Hold stop bit one and a half cycle
            DELAY UART_Tx_DT*3/2 - D'5' - TXLATENCY
        else                                ;Hold stop bit one cycle
            DELAY UART_Tx_DT - D'5' - TXLATENCY
        endif
    endif
    bsf INTCON, GIE                         ;Interrupts back on if enabled
    retlw 0
endif           ;End ifdef
