;==============================================================================
;*Author:  Jonathan Weaver, jonw0224@netscape.net
;E-mail Contact: jonw0224@netscape.net
;*Description:  Header information for the software (bitbang) serial port
;*Version: 1.0
;*Date: 2/16/2005
;*Filename: serialh.inc
;
;Copyright (C) 2005 Jonathan Weaver
;
;This program is free software; you can redistribute it and/or modify it under
;the terms of the GNU General Public License as published by the Free Software
;Foundation; either version 2 of the License, or (at your option) any later
;version.
;
;This program is distributed in the hope that it will be useful, but WITHOUT
;ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
;FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
;details.
;
;You should have received a copy of the GNU General Public License along with
;this program; if not, write to the Free Software Foundation, Inc.,
;51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
;
;Revision History:
; v1.0 - 2/16/2005 - Created file
;
;==============================================================================

;*TXPORT is a literal that determines the transmit port
;external TXPORT

;*TXPIN is a literal that determines the transmitting pin on the transmit port
;external TXPIN

;*RXPORT is a literal that determines the recieve port
;external RXPORT

;*RXPIN is a literal that determines the recieving pin on the recieve port
;external RXPIN

;*SERIAL_CONF is a literal that determines the configuration of the serial port.  The following constants are declarations that aid in assigning SERIAL_CONF.  <br><br>NEGPOLARITY equ 1 << 7.  If bit seven is set then a logic zero is +5V on the serial port and logic one is 0V.  <br><br>PARITYON equ 1 << 6. If bit six is set then a parity bit is added to each eight bit word and parity checks are implemented.  <br><br>STICKYPARITY equ 1 << 5.  If bit five is set then sticky parity is used.  Sticky parity means that the parity bit is either one or zero all the time.  <br><br>ODDPARITY equ 1 << 4. If bit four is set then odd parity is used.  Otherwise even parity is used.  Odd parity sets the parity bit so that the total number of ones in the nine bit word is odd. Even parity sets the parity bit so that the total number of ones in the nine bit word is even.  If STICKYPARITY is used, odd parity means that the parity bit is always one and even parity means that the parity bit is always zero.  <br><br>RXINTERRUPT equ 1 << 3.  If bit 3 is set, then recieve should be interrupt Driven.  Setting bit 3 prevents the software from polling for a start bit before beginning to recieve.  <br><br>STOPONEANDHALF equ 2 Stop bit lasts for 1.5 bit cycle time.  <br><br>STOPTWO equ 1.  Stop bit lasts for 2 bit cycle time.  <br><br>For example, if you wanted no parity bit, with positive polarity then you would set SERIAL_CONF equ 0.  <br><br>If you wanted an odd parity bit, with negative polarity then you would set SERIAL_CONF equ NEGPOLARITY + PARITYON + ODDPARITY
;external SERIAL_CONF

;The following constants are declarations that aid in assigning SERIAL_CONF
NEGPOLARITY equ 1 << 7      ;If bit seven is set then a logic zero is +5V on
                            ;the serial port and logic one is 0V
PARITYON equ 1 << 6         ;If bit six is set then a parity bit is added to
                            ;each eight bit word and parity checks are
                            ;implemented
STICKYPARITY equ 1 << 5     ;If bit five is set then sticky parity is used.
                            ;Sticky parity means that the parity bit is either
                            ;one or zero al the time
ODDPARITY equ 1 << 4        ;If bit four is set then odd parity is used.
                            ;Otherwise even parity is used.  Odd parity sets the
                            ;parity bit so that the total number of ones in the
                            ;nine bit word is odd. Even parity sets
                            ;the parity bit so that the total number of ones
                            ;in the nine bit word is even.  If STICKYPARITY is
                            ;used, odd parity means that the parity bit is
                            ;always one and even parity means that the parity
                            ;bit is always zero.
RXINTERRUPT equ 1 << 3      ;If bit 3 is set, then recieve should be interrupt
                            ;Driven.  Setting bit 3 prevents the software from
                            ;polling for a start bit before beginning to
                            ;recieve.
STOPONEANDHALF equ 2        ;Stop bit lasts for 1.5 bit cycle time
STOPTWO equ 1               ;Stop bit lasts for 2 bit cycle time

;For example, if you wanted no parity bit, with positive polarity then
;you would set SERIAL_CONF equ 0.
;If you wanted an odd parity bit, with negative polarity then you would set
;SERIAL_CONF equ NEGPOLARITY + PARITYON + ODDPARITY

;*RXLATENCY is a literal that specifies the latency (in cycles) from the startof an interrupt to the call of UART_Rx.  RXLATENCY should include 2 cycles forthe call statement of UART_Rx.  If interrupts are not used RXLATENCY should bezero.
;external RXLATENCY

;*TXLATENCY is a literal that specifies the latency (in cycles) from the end of an transmit call to the beginning of the next closest transmit call.TXLATENCY is used to shorten the stop bit and allow the software to processthe next byte (while holding Tx high) before sending the start bit of the nextbyte.  TXLATENCY should include 2 cycles for the call statement of UART_Tx.
;external TXLATENCY

;*BAUDRATE is a literal that determines the communications rate.  Simple set the BAUDRATE to the desired communications rate in Hz.
;external BAUDRATE

;*CLOCKFREQ is a literal that determines the clock frequency.  Simply set the CLOCKFREQ to the used clock frequency in kHz.  For example, if the chip is using a 20 MHz crystal, set CLOCKFREQ equ 20000.
;external CLOCKFREQ

;*FEF and PEF are declarations defined in serialh.inc that aid in decifering the status code returned by UART_Rx or UART_Tx.  <br><br>FEF equ 1 << 1 is a framing error.  <br><br>PEF equ 1 << 0 is a parity Error
;external FEF & PEF

FEF equ 1 << 1              ;Framing Error
PEF equ 1 << 0              ;Parity Error

;------------------------------------------------------------------------------
TXONE macro
    if (SERIAL_CONF & NEGPOLARITY) == B'10000000'
        bcf TXPORT, TXPIN
    else
        bsf TXPORT, TXPIN
    endif
endm

;------------------------------------------------------------------------------
TXZERO macro
    if (SERIAL_CONF & NEGPOLARITY) == B'10000000'
        bsf TXPORT, TXPIN
    else
        bcf TXPORT, TXPIN
    endif
endm

;------------------------------------------------------------------------------
RXSKIPIFONE macro
    if (SERIAL_CONF & NEGPOLARITY) == B'10000000'
        btfsc RXPORT, RXPIN
    else
        btfss RXPORT, RXPIN
    endif
endm

;------------------------------------------------------------------------------
RXSKIPIFZERO macro
    if (SERIAL_CONF & NEGPOLARITY) == B'10000000'
        btfss RXPORT, RXPIN
    else
        btfsc RXPORT, RXPIN
    endif
endm


;------------------------------------------------------------------------------
;*Initializes the software UART
UART_Initialize macro	;public
    BANK1
    ifdef _UART_Rx
        bsf RXPORT, RXPIN
    endif
    ifdef _UART_Tx
        bcf TXPORT, TXPIN
        BANK0
        TXONE
    else
        BANK0
    endif
endm
